@@ -41,14 +41,13 @@
 
 /* map configuration: */
 #define ENABLE_OSD
-#undef OSD_BALLOON
 #define OSD_SCALE
 #define OSD_CROSSHAIR
 #define OSD_COORDINATES
 #undef OSD_NAV
 
 #ifdef USE_MAEMO
-#define OSM_GPS_MAP_DRAG_LIMIT    (32)
+#define MAP_DRAG_LIMIT            (32)
 
 #define OSD_FONT_SIZE             (28.0)
 #define OSD_DIAMETER              (60)
@@ -65,6 +64,7 @@
 #define MAP_KEY_ZOOMIN      HILDON_HARDKEY_INCREASE
 #define MAP_KEY_ZOOMOUT     HILDON_HARDKEY_DECREASE
 #else
+#define MAP_DRAG_LIMIT      (10)
 #define MAP_KEY_FULLSCREEN  GDK_F11
 #define MAP_KEY_ZOOMIN      '+'
 #define MAP_KEY_ZOOMOUT     '-'
@@ -78,13 +78,15 @@
 /* specify OSD colors explicitely. Otherwise gtk default */
 /* colors are used. fremantle always uses gtk defaults */
 #ifndef MAEMO5
-#define OSD_COLOR_BG         1, 1, 1         // white background
+#define OSD_COLOR_BG         1, 1, 1, 1      // white background
 #define OSD_COLOR            0.5, 0.5, 1     // light blue border and controls
 #define OSD_COLOR_DISABLED   0.8, 0.8, 0.8   // light grey disabled controls
 #define OSD_SHADOW_ENABLE
 #else
+#define OSD_COLOR_BG         0, 0, 0, 0.5    // transparent dark background
 /* fremantle has controls at botton (for fringer friendlyness) */
 #define OSD_Y  -10
+#define OSD_HR_Y 10    // HR is always at screens top
 #endif
 
 #define OSD_DOUBLE_BUFFER    // render osd/map together offscreen
@@ -92,5 +94,7 @@
 #define OSD_NO_DPAD          // no direction arrows (map is panned)
 #define OSD_SOURCE_SEL       // display source selection tab
 #define OSD_BALLOON
+#define OSD_DOUBLEPIXEL      // allow pixel doubling from OSD
+#define OSD_HEARTRATE
 
 #endif // CONFIG_H
