@@ -36,6 +36,10 @@
 #include <gpsmgr.h>
 #endif
 #include <errno.h>
+#else
+#ifdef USE_LIBGPS
+#include <gps.h>
+#endif
 #endif
 
 #include "gps.h"
@@ -50,12 +54,12 @@ static void gps_unregister_all(gps_state_t *gps_state);
 #define ALTITUDE_PRECISION (1.0)        // 1 meters
 
 static gboolean 
-track_differs(struct gps_fix_t *fix1, struct gps_fix_t *fix2) {
+track_differs(struct gps_t *fix1, struct gps_t *fix2) {
   return( fabsf(fix1->track - fix2->track) >= TRACK_PRECISION);
 }
 
 static gboolean 
-latlon_differs(struct gps_fix_t *fix1, struct gps_fix_t *fix2) {
+latlon_differs(struct gps_t *fix1, struct gps_t *fix2) {
   if( fabsf(fix1->latitude - fix2->latitude) >= POS_PRECISION)
     return TRUE;
 
@@ -66,12 +70,12 @@ latlon_differs(struct gps_fix_t *fix1, struct gps_fix_t *fix2) {
 }
 
 static gboolean 
-eph_differs(struct gps_fix_t *fix1, struct gps_fix_t *fix2) {
+eph_differs(struct gps_t *fix1, struct gps_t *fix2) {
   return( fabsf(fix1->eph - fix2->eph) >= EPH_PRECISION);
 }
 
 static gboolean 
-altitude_differs(struct gps_fix_t *fix1, struct gps_fix_t *fix2) {
+altitude_differs(struct gps_t *fix1, struct gps_t *fix2) {
   return( fabsf(fix1->altitude - fix2->altitude) >= ALTITUDE_PRECISION);
 }
 
@@ -92,26 +96,34 @@ static void gps_cb_func(gpointer data, gpointer user_data) {
 
   /* check for changes in position */
   if(callback->mask & LATLON_CHANGED &&
-     (((gps_state->set & LATLON_SET) != (gps_state->last.set & LATLON_SET)) ||
-      ((gps_state->set & LATLON_SET) && latlon_differs(&gps_state->last.fix, &gps_state->fix)))) 
+     (((gps_state->set & FIX_LATLON_SET) != 
+       (gps_state->last.set & FIX_LATLON_SET)) ||
+      ((gps_state->set & FIX_LATLON_SET) && 
+       latlon_differs(&gps_state->last.fix, &gps_state->fix)))) 
     gps_state->set |= LATLON_CHANGED;
 
   /* check for changes in eph */
   if(callback->mask & HERR_CHANGED &&
-     (((gps_state->set & HERR_SET) != (gps_state->last.set & HERR_SET)) ||
-      ((gps_state->set & HERR_SET) && eph_differs(&gps_state->last.fix, &gps_state->fix))))
+     (((gps_state->set & FIX_HERR_SET) != 
+       (gps_state->last.set & FIX_HERR_SET)) ||
+      ((gps_state->set & FIX_HERR_SET) && 
+       eph_differs(&gps_state->last.fix, &gps_state->fix))))
     gps_state->set |= HERR_CHANGED;
 
   /* check for changes in track */
   if(callback->mask & TRACK_CHANGED &&
-     (((gps_state->set & TRACK_SET) != (gps_state->last.set & TRACK_SET)) ||
-      ((gps_state->set & TRACK_SET) && track_differs(&gps_state->last.fix, &gps_state->fix))))
+     (((gps_state->set & FIX_TRACK_SET) != 
+       (gps_state->last.set & FIX_TRACK_SET)) ||
+      ((gps_state->set & FIX_TRACK_SET) && 
+       track_differs(&gps_state->last.fix, &gps_state->fix))))
     gps_state->set |= TRACK_CHANGED;
 
   /* check for changes in altitude */
   if(callback->mask & ALTITUDE_CHANGED &&
-     (((gps_state->set & ALTITUDE_SET) != (gps_state->last.set & ALTITUDE_SET)) ||
-      ((gps_state->set & ALTITUDE_SET) && altitude_differs(&gps_state->last.fix, &gps_state->fix))))
+     (((gps_state->set & FIX_ALTITUDE_SET) != 
+       (gps_state->last.set & FIX_ALTITUDE_SET)) ||
+      ((gps_state->set & FIX_ALTITUDE_SET) && 
+       altitude_differs(&gps_state->last.fix, &gps_state->fix))))
     gps_state->set |= ALTITUDE_CHANGED;
 
   if(gps_state->set & CHANGED_MASK)
@@ -120,6 +132,27 @@ static void gps_cb_func(gpointer data, gpointer user_data) {
 
 #ifndef ENABLE_LIBLOCATION
 
+static gboolean gps_notify(gpointer data) {
+  gps_state_t *gps_state = (gps_state_t*)data;
+
+  if(gps_state->callbacks) {
+    g_mutex_lock(gps_state->mutex);
+
+    /* tell all clients */
+    g_slist_foreach(gps_state->callbacks, gps_cb_func, gps_state);
+
+    g_mutex_unlock(gps_state->mutex);
+  }
+
+  /* remember last state reported */
+  gps_state->last.set = gps_state->set;
+  gps_state->last.fix = gps_state->fix;
+  
+  return FALSE; 
+}
+
+#ifndef USE_LIBGPS
+
 /* maybe user configurable later on ... */
 #define GPSD_HOST "127.0.0.1"
 #define GPSD_PORT 2947
@@ -200,6 +233,7 @@ static double parse_double(char *val) {
 /* unpack a daemon response into a status structure */
 static void gps_unpack(char *buf, gps_state_t *gps_state) {
   char *ns, *sp, *tp;
+  int j;
 
   for(ns = buf; ns; ns = strstr(ns+1, "GPSD")) {
     if(strncmp(ns, "GPSD", 4) == 0) {
@@ -218,7 +252,7 @@ static void gps_unpack(char *buf, gps_state_t *gps_state) {
 	    gps_state->fix.track = NAN;
 	    gps_state->fix.eph = NAN;
 	  } else {
-	    struct gps_fix_t nf;
+	    struct gps_t nf;
 	    char tag[MAXTAGLEN+1], alt[20];
 	    char eph[20], track[20],speed[20];
 	    char lat[20], lon[20];
@@ -236,44 +270,51 @@ static void gps_unpack(char *buf, gps_state_t *gps_state) {
 	      nf.track = parse_double(track);
 
 	      if (!isnan(nf.eph))
-		gps_state->set |= HERR_SET;
+		gps_state->set |= FIX_HERR_SET;
 
 	      if (!isnan(nf.track))
-		gps_state->set |= TRACK_SET;
+		gps_state->set |= FIX_TRACK_SET;
 
 	      gps_state->fix = nf;
-	      gps_state->set |= LATLON_SET;
+	      gps_state->set |= FIX_LATLON_SET;
 
 	      if(!isnan(nf.altitude))
-		gps_state->set |= ALTITUDE_SET;
+		gps_state->set |= FIX_ALTITUDE_SET;
 	    }
 	  }
 	  break;
+
+        case 'Y': 
+	  gps_state->fix.sat_num = 0;
+
+          if (sp[2] != '?') {
+            (void)sscanf(sp, "Y=%*s %*s %d ", &gps_state->fix.sat_num);
+
+	    /* clear all slots */
+            for (j = 0; j < gps_state->fix.sat_num; j++) 
+	      gps_state->fix.sat_data[j].prn = 
+	      gps_state->fix.sat_data[j].ss = 
+	      gps_state->fix.sat_data[j].used = 0;
+
+	    printf("gps: sats = %d\n", gps_state->fix.sat_num);
+            for (j = 0; j < gps_state->fix.sat_num; j++) {
+              if ((sp != NULL) && ((sp = strchr(sp, ':')) != NULL)) {
+                sp++;
+                (void)sscanf(sp, "%d %*d %*d %d %d", 
+			     &gps_state->fix.sat_data[j].prn,
+			     &gps_state->fix.sat_data[j].ss, 
+			     &gps_state->fix.sat_data[j].used);
+              }
+            }
+          }
+          gps_state->set |= FIX_SATELLITE_SET;
+	  break;
 	}
       }
     }
   }
 }
 
-static gboolean gps_notify(gpointer data) {
-  gps_state_t *gps_state = (gps_state_t*)data;
-
-  if(gps_state->callbacks) {
-    g_mutex_lock(gps_state->mutex);
-
-    /* tell all clients */
-    g_slist_foreach(gps_state->callbacks, gps_cb_func, gps_state);
-
-    g_mutex_unlock(gps_state->mutex);
-  }
-
-  /* remember last state reported */
-  gps_state->last.set = gps_state->set;
-  gps_state->last.fix = gps_state->fix;
-  
-  return FALSE; 
-}
-
 static gpointer gps_thread(gpointer data) {
   gps_state_t *gps_state = (gps_state_t*)data;
   
@@ -281,13 +322,16 @@ static gpointer gps_thread(gpointer data) {
   GnomeVFSResult vfs_result;
   char str[512];
 
-  const char *msg = "o\r\n";   /* pos request */
-
   gps_state->set = 0;
 
   gboolean connected = FALSE;
 
   while(1) {
+    /* just lock and unlock the control mutex. This stops the thread */
+    /* while the main process locks this mutex */
+    g_mutex_lock(gps_state->control_mutex);
+    g_mutex_unlock(gps_state->control_mutex);
+
     if(!connected) {
       printf("gps: trying to connect\n");
 
@@ -295,7 +339,10 @@ static gpointer gps_thread(gpointer data) {
 	sleep(10);
       else 
 	connected = TRUE;
+
     } else {
+      const char *msg = "o\r\n";   /* pos request */
+
       if(GNOME_VFS_OK == 
 	 (vfs_result = gnome_vfs_socket_write(gps_state->socket,
 		      msg, strlen(msg)+1, &bytes_read, NULL))) {
@@ -307,6 +354,7 @@ static gpointer gps_thread(gpointer data) {
 	if(bytes_read == (strlen(msg)+1)) {
 	  vfs_result = gnome_vfs_socket_read(gps_state->socket,
 	     str, sizeof(str)-1, &bytes_read, NULL);
+
 	  if(vfs_result == GNOME_VFS_OK) {
 	    str[bytes_read] = 0; 
 
@@ -336,6 +384,7 @@ gps_state_t *gps_init(void) {
 
   /* start a new thread to listen to gpsd */
   gps_state->mutex = g_mutex_new();
+  gps_state->control_mutex = g_mutex_new();
   gps_state->thread_p = 
     g_thread_create(gps_thread, gps_state, FALSE, NULL);
 
@@ -348,9 +397,100 @@ void gps_release(gps_state_t *gps_state) {
 #ifdef USE_MAEMO
   gpsbt_stop(&gps_state->context);
 #endif
+
   g_free(gps_state);
 }
 
+#else // USE_LIBGPS
+
+static gpointer gps_thread(gpointer data) {
+  gps_state_t *gps_state = (gps_state_t*)data;
+
+  /* the following is required for libgps to be able to parse */
+  /* the gps messages. Unfortunately this also affect the rest of */
+  /* the program and the main thread */
+  setlocale(LC_NUMERIC, "C");
+
+  while(1) {
+    /* just lock and unlock the control mutex. This stops the thread */
+    /* while the main process locks this mutex */
+    g_mutex_lock(gps_state->control_mutex);
+    g_mutex_unlock(gps_state->control_mutex);
+
+    gps_poll(gps_state->data);
+
+    g_mutex_lock(gps_state->mutex);
+    
+    /* assume we could't parse anything ... */
+    gps_state->set = 0;
+    
+    if(gps_state->data->fix.mode >= MODE_2D) {
+      /* latlon valid */
+      gps_state->set |= FIX_LATLON_SET;
+      gps_state->fix.latitude = gps_state->data->fix.latitude;
+      gps_state->fix.longitude = gps_state->data->fix.longitude;
+
+      gps_state->fix.eph = 
+	gps_state->data->fix.epy > gps_state->data->fix.epx ?
+	gps_state->data->fix.epy : gps_state->data->fix.epx;
+      if(!isnan(gps_state->fix.eph)) gps_state->set |= FIX_HERR_SET;
+
+      gps_state->fix.track = gps_state->data->fix.track;
+      if(!isnan(gps_state->fix.track)) gps_state->set |= FIX_TRACK_SET;
+    }
+
+    if(gps_state->data->fix.mode >= MODE_3D) {
+      /* altitude valid */
+      gps_state->set |= FIX_ALTITUDE_SET;
+      gps_state->fix.altitude = gps_state->data->fix.altitude;
+    } else
+      gps_state->fix.altitude = NAN;
+
+    /* notify applications of state if useful */
+    g_idle_add(gps_notify, gps_state); 
+    
+    g_mutex_unlock(gps_state->mutex);	    
+    
+  }
+
+  printf("gps: thread ended???\n");
+  return NULL;
+}
+
+gps_state_t *gps_init(void) {
+  gps_state_t *gps_state = g_new0(gps_state_t, 1);
+
+  gps_state->mutex = g_mutex_new();
+  gps_state->control_mutex = g_mutex_new();
+
+  gps_state->data = gps_open("127.0.0.1", DEFAULT_GPSD_PORT );
+  if(!gps_state->data)
+    perror("gps_open()");
+
+  (void)gps_stream(gps_state->data, WATCH_ENABLE, NULL);
+
+  gps_state->thread_p = 
+    g_thread_create(gps_thread, gps_state, FALSE, NULL);
+
+  return gps_state;
+}
+
+void gps_release(gps_state_t *gps_state) { 
+  gps_unregister_all(gps_state);
+  gps_close(gps_state->data); 
+  g_free(gps_state);
+}
+
+#endif // USE_LIBGPS
+
+static void gps_background_enable(gps_state_t *gps_state, gboolean enable) {
+  printf("GPS: %sable background process\n", enable?"en":"dis");
+
+  /* start and stop gps thread by locking and unlocking the control mutex */
+  if(enable) g_mutex_unlock(gps_state->control_mutex);
+  else       g_mutex_lock(gps_state->control_mutex);
+}
+
 #else
 
 static void
@@ -359,20 +499,20 @@ location_changed(LocationGPSDevice *device, gps_state_t *gps_state) {
   gps_state->set = 0;
 
   if(device->fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET) { 
-    gps_state->set |= LATLON_SET | HERR_SET;
+    gps_state->set |= FIX_LATLON_SET | FIX_HERR_SET;
     gps_state->fix.latitude = device->fix->latitude;
     gps_state->fix.longitude = device->fix->longitude;
     gps_state->fix.eph = device->fix->eph/100.0;  // we want eph in meters
   }
 
   if(device->fix->fields & LOCATION_GPS_DEVICE_ALTITUDE_SET) {
-    gps_state->set |= ALTITUDE_SET;
+    gps_state->set |= FIX_ALTITUDE_SET;
     gps_state->fix.altitude = device->fix->altitude;
   } else
     gps_state->fix.altitude = NAN;
 
   if(device->fix->fields & LOCATION_GPS_DEVICE_TRACK_SET) {
-    gps_state->set |= TRACK_SET;
+    gps_state->set |= FIX_TRACK_SET;
     gps_state->fix.track = device->fix->track;
   }
 
@@ -397,6 +537,7 @@ gps_state_t *gps_init(void) {
   gps_state->idd_changed = 
     g_signal_connect(gps_state->device, "changed", 
 		     G_CALLBACK(location_changed), gps_state);
+  gps_state->connected = TRUE;
 
   gps_state->control = location_gpsd_control_get_default();
 
@@ -426,27 +567,57 @@ void gps_release(gps_state_t *gps_state) {
   
   /* Disconnect signal */
   g_signal_handler_disconnect(gps_state->device, gps_state->idd_changed);
+  gps_state->connected = FALSE;
   
   g_free(gps_state);
 }
 
-#endif // USE_LIBLOCATION
+static void gps_background_enable(gps_state_t *gps_state, gboolean enable) {
+  printf("GPS: %sable background process\n", enable?"en":"dis");
 
-void gps_register_callback(gps_state_t *gps_state, int mask, 
-			   gps_cb cb, void *data) {
-  gps_callback_t *callback = g_new0(gps_callback_t, 1);
-
-  callback->mask = mask;
-  callback->cb = cb;
-  callback->data = data;
-
-  gps_state->callbacks = g_slist_append(gps_state->callbacks, callback);
+  if(enable) {
+    if(!gps_state->connected) {
+      gps_state->idd_changed = 
+	g_signal_connect(gps_state->device, "changed", 
+			 G_CALLBACK(location_changed), gps_state);
+      gps_state->connected = TRUE;
+    }
+  } else {
+    if(gps_state->connected) {
+      /* Disconnect signal */
+      g_signal_handler_disconnect(gps_state->device, gps_state->idd_changed);
+      gps_state->connected = FALSE;
+    }
+  }
 }
 
+#endif // USE_LIBLOCATION
+
 static gint compare(gconstpointer a, gconstpointer b) {
   return ((gps_callback_t*)a)->cb != b;
 }
 
+void gps_register_callback(gps_state_t *gps_state, int mask, 
+			   gps_cb cb, void *data) {
+  /* make sure the callback isn't already registered */
+  GSList *list = g_slist_find_custom(gps_state->callbacks, cb, compare);
+  if(list) {
+    printf("GPS register: ignoring duplicate\n"); 
+    return;
+  }
+
+  gps_callback_t *callback = g_new0(gps_callback_t, 1);
+  
+  callback->mask = mask;
+  callback->cb = cb;
+  callback->data = data;
+  
+  gps_state->callbacks = g_slist_append(gps_state->callbacks, callback);
+  
+  if(g_slist_length(gps_state->callbacks) == 1)
+    gps_background_enable(gps_state, TRUE);
+}
+
 void gps_unregister_callback(gps_state_t *gps_state, gps_cb cb) {
   /* find callback in list */
   GSList *list = g_slist_find_custom(gps_state->callbacks, cb, compare);
@@ -455,6 +626,9 @@ void gps_unregister_callback(gps_state_t *gps_state, gps_cb cb) {
   /* and de-chain and free it */
   g_free(list->data);
   gps_state->callbacks = g_slist_remove(gps_state->callbacks, list->data);
+
+  if(g_slist_length(gps_state->callbacks) == 0)
+    gps_background_enable(gps_state, FALSE);
 }
 
 static void gps_unregister_all(gps_state_t *gps_state) {
